"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const json_loader_1 = require("./json-loader");
class DepDiff {
    constructor(options) {
        this.options = options;
    }
    diff(pckg) {
        return {
            dependencies: this.options.dependencies ? this.parseDependencies(pckg.dependencies) : null,
            devDependencies: this.options.devDependencies ? this.parseDependencies(pckg.devDependencies) : null,
            peerDependencies: this.options.peerDependencies ? this.parseDependencies(pckg.peerDependencies) : null,
            optionalDependencies: this.options.optionalDependencies ? this.parseDependencies(pckg.optionalDependencies) : null
        };
    }
    parseDependencies(dependencies) {
        const dependencyVersions = this.readVersions(dependencies);
        const installedDependencyVersions = dependencyVersions
            .map(x => this.readInstalledVersion(x.dependency));
        return this.compare(dependencyVersions, installedDependencyVersions)
            .filter(dependency => !this.equalVersion(dependency.listed, dependency.installed));
    }
    readVersions(dependencies) {
        return dependencies ? Object
            .keys(dependencies)
            .map(key => ({ dependency: key, version: dependencies[key] })) : [];
    }
    readInstalledVersion(dependency) {
        return {
            dependency,
            version: json_loader_1.JsonLoader.load(`node_modules/${dependency}/package.json`).version
        };
    }
    equalVersion(listed, installed) {
        const regex = new RegExp('(\^|\~)?(' + installed + ')', 'g');
        return regex.exec(listed);
    }
    compare(dependencyVersions, installedDependencyVersions) {
        return dependencyVersions
            .map(x => {
            const installedVersion = installedDependencyVersions
                .find(y => y.dependency === x.dependency);
            return {
                dependency: x.dependency,
                listed: x.version,
                installed: installedVersion.version
            };
        });
    }
}
exports.DepDiff = DepDiff;
//# sourceMappingURL=dep-diff.js.map